%% Reproduce results of Figures 4 and 6 in the follwing paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Departments of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG.
clc;
clear all;
close all;
clear classes;

%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% generate synthetic data and mask
load SyntheticData_64x64_10Peaks
load pseudoPoissonGapMask_64x64_SR_020

[signal_row, signal_col] = size(FID);
hybridData = fft(FID,[],2);
normalizedFactor = max(abs(hybridData(:))); 
hybridData = hybridData / normalizedFactor; % normalization

%% display the fully sampled spectrum
originalSpectrum = fft2(FID);
contourLevel = 10;
figure, contour( abs(originalSpectrum/max(originalSpectrum(:))), contourLevel ); colorbar;
figure, contour( abs(hybridData/max(hybridData(:))), contourLevel ); colorbar;

%% add noise
noiseSTD = max( max(abs(hybridData)) )*0.01;                                   
noise = noiseSTD * ( randn( signal_row,signal_col ) + 1i*randn( signal_row, signal_col ) );
hybridData_noisy = hybridData + noise;

%% perform sparse sampling
% sampingRatio = 0.2;
% mask = Generate_2D_HTFPlanePoissonMask(sampingRatio,signal_row,signal_col);
% mask = flipud(mask);    % This is important!
zerosFillingSampledSignal = zeros(signal_row,signal_col);
index = find(mask==1);
zerosFillingSampledSignal(index) = hybridData_noisy(index);

figure,imshow(mask);
figure,contour(abs(zerosFillingSampledSignal/max(zerosFillingSampledSignal(:))), contourLevel);

%% Low Rank Block Hankel Matrix (LRBHM) reconstruction of hybrid time and frequency signals 
LRBHM_Params.tol = 1*1e-3;
LRBHM_Params.lambda = 1e5;

t_start = clock;
LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,mask,LRBHM_Params);
% LRBHM_RecG = Solver_HybridSignalReconstruction_LRBHM_HTF_simple(zerosFillingSampledSignal,mask,LRBHM_Params);
t_end = clock;
LRBHM_RunTime = etime(t_end,t_start);
LRBHM_RecSpectrum = fft(LRBHM_RecHybridData,[],1);

%% Compressed Sensing (CS) reconstruction
CS_Params.tol = 1e-4;
CS_Params.lambda = 1e5;

t_start = clock;
CS_RecSpectrum = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,mask,CS_Params);
t_end = clock;
CS_RunTime = etime(t_end,t_start);

%% show reconstruction spectra
figure, contour( abs(LRBHM_RecSpectrum/max(abs(LRBHM_RecSpectrum(:)))), contourLevel ); title('LRBHM reconstruction');
figure, contour( abs(CS_RecSpectrum/max(abs(CS_RecSpectrum(:)))), contourLevel ); title('CS reconstruction');

%% peaks intensity correlation
threshold = 0.1;
peaksCorr(originalSpectrum./max(abs(originalSpectrum(:))), CS_RecSpectrum./max(abs(CS_RecSpectrum(:))),threshold,'r');
peaksCorr(originalSpectrum./max(abs(originalSpectrum(:))), LRBHM_RecSpectrum./max(abs(LRBHM_RecSpectrum(:))),threshold,'b');

%%
% save LRBHM_vs_CS_SyntheticData_64x64_10Peaks.mat;
%% ----------------- The end of the file ----------------------------------