% Demo for generating 1D or 2D time domain signal (FID) of spectra
% Authors: Jiaxi Ying, Xiaobo Qu 
% Date: 2017/10/24
% Corresponding Author: Xiaobo Qu
% Homepage: http://csrc.xmu.edu.cn
% Email: quxiaobo <|at|> xmu.edu.cn
% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
% All rights are reserved by CSG.
%%
clear;
close all;
clc;
%% Generate the time domain data of 1D spectrum
numPoints = 512;
numPeaks = 5;
phase = zeros(numPeaks,1);
amp = ones(numPeaks,1);                 % amplitude
omga = [0.1 0.3 0.5 0.7 0.9];           % frequencies
tao = 16+64*[0.1 0.6 1.1 1.6 2.1];      % damping factor
[fid,t] = genrateFID_1D_Peaks(amp,omga,phase,tao,numPoints);     
FID = fid';
FID = FID / max( abs(FID) );
originalSpectrum = fft(FID);
normalizeOriginalSpectra = originalSpectrum./max(abs(originalSpectrum(:)));
figure,plot(abs(normalizeOriginalSpectra),'Color',[0.47 0.67 0.19],'LineWidth',3);
%% Generate the time domain data of 2D spectrum
NumPeaks=20;
NumPoints_row=64;
NumPoints_col=64;
Amp_Relative=[1 1 1 1 1 1 1 1 1 1 1];
Omga_row=[10 10 40 20 30 50 20 30 40 50 60]/64;
Omga_col=[10 20 40 10 30 50 20 50 30 40 60]/64;
Tao_row=rand(1,11)*32+16;
Tao_col=rand(1,11)*32+16;
fid2D=genrateFID_2D(Amp_Relative,Omga_row,Omga_col,Tao_row,Tao_col,NumPoints_row,NumPoints_col);
spectra2D=fft2(fid2D)/sqrt(length(fid2D(:)));
figure,contour(flipud(abs(spectra2D)),10),colorbar;
figure,mesh(1:size(fid2D,2),1:size(fid2D,1),abs(flipud(spectra2D)));