%% Reproduce Table II (Quantitative Analysis of STEU COSY) in the following paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG. 
clc;
clear all;
close all;
clear classes;
%% record time and create log file
% programStartTime = datestr(now);
% diary('./log/LRBHM_10Times_log.txt');
% diary on;
%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% load data and mask
load marrow_cosy

hybridData = dataO;
[signal_row,signal_col] = size(hybridData);
normalizedFactor = max(abs(hybridData(:)));
hybridData = hybridData/normalizedFactor;

%% LRBHM-HTF
samplingRate = 0.20;
repeatedNum = 10;
LRBHM_RunTime = zeros(1,repeatedNum);
allMask = zeros(signal_row,signal_col,repeatedNum);
LRBHM_RecFID = zeros(signal_row,signal_col,repeatedNum);

LRBHM_params.tol = 5e-3;
LRBHM_params.lambda = 1e5;

for cnt = 1:1:repeatedNum
    % down sampling
    currentMask = Generate_2D_HTFPlanePoissonMask(samplingRate,signal_row,signal_col);
    currentMask = flipud(currentMask);  % mask must be flipped to be consistent for data !!!

    allMask(:,:,cnt) = currentMask;
    zerosFillingSampledSignal = zeros(signal_row,signal_col);
    index = find(currentMask==1);
    zerosFillingSampledSignal(index) = hybridData(index);
    
    % reconstruction
    fprintf('\n Start runing LRBHM reconstruction (%dth loop time) \n', cnt);
    t_start = clock;
    LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,currentMask,LRBHM_params);
    t_end = clock;
    LRBHM_RecFID(:,:,cnt) = ifft(LRBHM_RecHybridData,[],2);
    LRBHM_RunTime(1, cnt) = etime(t_end,t_start);
    fprintf('--> Finished running LRBHM reconstruction, used time: %d! (%dth loop time) \n', LRBHM_RunTime(1,cnt), cnt);
end

%% Spectra with sparsity constraint (CS)
CS_Params.lamda = 1e5;
CS_Params.tol = 1e-3;

CS_RunTime = zeros(1,repeatedNum);
CS_RecSpec = zeros(signal_row,signal_col,repeatedNum);

for cnt = 1:1:repeatedNum
    % down sampling
    currentMask = allMask(:,:,cnt); % use the same mask as LRBHM
    zerosFillingSampledSignal = zeros(signal_row,signal_col);
    index = find(currentMask==1);
    zerosFillingSampledSignal(index) = hybridData(index);
    
    % reconstruction
    fprintf('\n Start runing CS reconstruction (%dth loop time) \n', cnt);
    t_start = clock;
    CS_RecSpec(:,:,cnt) = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,currentMask,CS_Params);
    t_end = clock;
    CS_RunTime(1, cnt) = etime(t_end,t_start);
    fprintf('--> Finished running CS reconstruction, used time: %d! (%dth loop time) \n', CS_RunTime(1,cnt), cnt);
end

%%
result = [];
for cnt = 1: repeatedNum
data = (CS_RecSpec(:,:,cnt));
data = ifft((data));
S2 = AddWindow(data,100,150,512,512,0.5,0.5,0,40);

area = [380,400,482,496]; S2s = S2(area(1):area(2),area(3):area(4)); result(1)=sum(sum(abs(S2s)));
area = [348,364,446,452]; S2s = S2(area(1):area(2),area(3):area(4)); result(2)=sum(sum(abs(S2s)));
area = [322,338,412,425]; S2s = S2(area(1):area(2),area(3):area(4)); result(3)=sum(sum(abs(S2s)));
area = [280,300,360,380]; S2s = S2(area(1):area(2),area(3):area(4)); result(4)=sum(sum(abs(S2s)));
area = [265,283,342,357]; S2s = S2(area(1):area(2),area(3):area(4)); result(5)=sum(sum(abs(S2s)));
area = [222,238,291,303]; S2s = S2(area(1):area(2),area(3):area(4)); result(6)=sum(sum(abs(S2s)));
area = [113,121,159,166]; S2s = S2(area(1):area(2),area(3):area(4)); result(7)=sum(sum(abs(S2s)));
area = [96,106,140,146];  S2s = S2(area(1):area(2),area(3):area(4)); result(8)=sum(sum(abs(S2s)));
area = [1,19,25,45];      S2s = S2(area(1):area(2),area(3):area(4)); result(9)=sum(sum(abs(S2s)));

result = result./sum(result);
CS_Quantification(cnt,:) = result;
end
CS_Quantification = CS_Quantification.';

% LRBHM-HTF
result = [];
for cnt = 1: repeatedNum
data = LRBHM_RecFID(:,:,cnt);
data = fft(data,[],2);
S2 = AddWindow(data,100,150,512,512,0.5,0.5,0,40);

area = [380,400,482,496]; S2s = S2(area(1):area(2),area(3):area(4)); result(1)=sum(sum(abs(S2s)));
area = [348,364,446,452]; S2s = S2(area(1):area(2),area(3):area(4)); result(2)=sum(sum(abs(S2s)));
area = [322,338,412,425]; S2s = S2(area(1):area(2),area(3):area(4)); result(3)=sum(sum(abs(S2s)));
area = [280,300,360,380]; S2s = S2(area(1):area(2),area(3):area(4)); result(4)=sum(sum(abs(S2s)));
area = [265,283,342,357]; S2s = S2(area(1):area(2),area(3):area(4)); result(5)=sum(sum(abs(S2s)));
area = [222,238,291,303]; S2s = S2(area(1):area(2),area(3):area(4)); result(6)=sum(sum(abs(S2s)));
area = [113,121,159,166]; S2s = S2(area(1):area(2),area(3):area(4)); result(7)=sum(sum(abs(S2s)));
area = [96,106,140,146];  S2s = S2(area(1):area(2),area(3):area(4)); result(8)=sum(sum(abs(S2s)));
area = [1,19,25,45];      S2s = S2(area(1):area(2),area(3):area(4)); result(9)=sum(sum(abs(S2s)));

result = result./sum(result);
LRBHM_Quantification(cnt,:) = result;
end
LRBHM_Quantification = LRBHM_Quantification.';

%% calculate values in Table II
CS_std = std(CS_Quantification.*100,1,2);
CS_ResonVolumn    = mean(CS_Quantification.*100,2);

[CS_ResonVolumn_sorted, index] = sort(CS_ResonVolumn,'descend');
CS_std_sorted = CS_std(index);

LRBHM_std = std(LRBHM_Quantification.*100,1,2);
LRBHM_ResonVolumn = mean(LRBHM_Quantification.*100,2);

LRBHM_ResonVolumn_sorted = sort(LRBHM_ResonVolumn,'descend');
LRBHM_std_sorted = LRBHM_std(index);

%% record end time
% programEndTime = datestr(now);
% diary off;

%% ----------------- The end of the file ----------------------------------